@@ -1,28 +1,22 @@
+# SPDX-License-Identifier: Apache-2.0
 # Copyright 2014-2016 The Meson development team
 
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-
-#     http://www.apache.org/licenses/LICENSE-2.0
-
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
+from __future__ import annotations
 
 from .vs2010backend import Vs2010Backend
 from ..mesonlib import MesonException
-from ..interpreter import Interpreter
-from ..build import Build
 import typing as T
 
+if T.TYPE_CHECKING:
+    from ..build import Build
+    from ..interpreter import Interpreter
 
 class Vs2013Backend(Vs2010Backend):
+
+    name = 'vs2013'
+
     def __init__(self, build: T.Optional[Build], interpreter: T.Optional[Interpreter]):
         super().__init__(build, interpreter)
-        self.name = 'vs2013'
         self.vs_version = '2013'
         self.sln_file_version = '12.00'
         self.sln_version_comment = '2013'
