@@ -1,6 +1,6 @@
 # SPDX-License-Identifier: Apache-2.0
 # Copyright 2017 The Meson development team
-# Copyright © 2023 Intel Corporation
+# Copyright © 2023-2024 Intel Corporation
 
 from __future__ import annotations
 
@@ -22,8 +22,9 @@ from dataclasses import dataclass
 from glob import glob
 from pathlib import Path
 from mesonbuild.environment import Environment, detect_ninja
-from mesonbuild.mesonlib import (MesonException, RealPathAction, get_meson_command, quiet_git,
-                                 windows_proof_rmtree, setup_vsenv, OptionKey)
+from mesonbuild.mesonlib import (GIT, MesonException, RealPathAction, get_meson_command, quiet_git,
+                                 windows_proof_rmtree, setup_vsenv)
+from .options import OptionKey
 from mesonbuild.msetup import add_arguments as msetup_argparse
 from mesonbuild.wrap import wrap
 from mesonbuild import mlog, build, coredata
@@ -79,7 +80,36 @@ def is_git(src_root: str) -> bool:
     Checks if meson.build file at the root source directory is tracked by git.
     It could be a subproject part of the parent project git repository.
     '''
-    return quiet_git(['ls-files', '--error-unmatch', 'meson.build'], src_root)[0]
+    if quiet_git(['ls-files', '--error-unmatch', 'meson.build'], src_root)[0]:
+        return True
+
+    if os.path.exists(os.path.join(src_root, '.git')):
+        msg = 'Source tree looks like it may be a git repo, '
+        if not GIT:
+            msg += 'but git is not installed!'
+            if 'GITLAB_CI' in os.environ:
+                msg += ' This is a gitlab bug.'
+        else:
+            msg += 'but git returned a failure. '
+            p, oe = quiet_git(['status'], src_root)
+            if 'dubious ownership' in oe:
+                # For a few years now, git has absolved itself of the responsibility to implement
+                # robust, safe software. Instead of detecting the signs of a problematic scenario,
+                # they have chosen to consider many legitimate and reasonable use cases as "dangerous",
+                # and implemented the number one threat to security worldwide: alert fatigue. Having
+                # done so, they then washed their hands of the matter and permanently tabled the
+                # notion of adding fine-grained detection. This is not just useless, it is *worse*
+                # than useless.
+                #
+                # In our case, the error is triply meaningless since we are already executing build
+                # system commands from the same directory. Either way, reject the notion that git is
+                # well designed or that its error messaging is a valid approach to the problem space.
+                msg += 'This is a bug in git itself, please set `git config --global safe.directory "*"`'
+            else:
+                msg += 'meson.build may not have been committed to git?'
+        mlog.warning(msg)
+    return False
+
 
 def is_hg(src_root: str) -> bool:
     return os.path.isdir(os.path.join(src_root, '.hg'))
@@ -222,7 +252,12 @@ class GitDist(Dist):
 class HgDist(Dist):
     def have_dirty_index(self) -> bool:
         '''Check whether there are uncommitted changes in hg'''
-        out = subprocess.check_output(['hg', '-R', self.src_root, 'summary'])
+        env = os.environ.copy()
+        env['LC_ALL'] = 'C'
+        # cpython's gettext has a bug and uses LANGUAGE to override LC_ALL,
+        # contrary to the gettext spec
+        env.pop('LANGUAGE', None)
+        out = subprocess.check_output(['hg', '-R', self.src_root, 'summary'], env=env)
         return b'commit: (clean)' not in out
 
     def create_dist(self, archives: T.List[str]) -> T.List[str]:
@@ -286,7 +321,7 @@ def run_dist_steps(meson_command: T.List[str], unpacked_src_dir: str, builddir:
         return 1
     return 0
 
-def check_dist(packagename: str, meson_command: ImmutableListProtocol[str], extra_meson_args: T.List[str], bld_root: str, privdir: str) -> int:
+def check_dist(packagename: str, _meson_command: ImmutableListProtocol[str], extra_meson_args: T.List[str], bld_root: str, privdir: str) -> int:
     print(f'Testing distribution package {packagename}')
     unpackdir = os.path.join(privdir, 'dist-unpack')
     builddir = os.path.join(privdir, 'dist-build')
@@ -300,6 +335,7 @@ def check_dist(packagename: str, meson_command: ImmutableListProtocol[str], extr
     unpacked_files = glob(os.path.join(unpackdir, '*'))
     assert len(unpacked_files) == 1
     unpacked_src_dir = unpacked_files[0]
+    meson_command = _meson_command.copy()
     meson_command += ['setup']
     meson_command += create_cmdline_args(bld_root)
     meson_command += extra_meson_args
